# frozen_string_literal: true

require "forwardable"

require "core/extension"

module Is
  # [public] Adds a global instance to any object.
  #
  module Global
    extend Is::Extension

    extends :definition do
      # [public] Return the current global instance.
      #
      def global
        @__global ||= new
      end

      # [public] Reset the global instance.
      #
      def reset
        remove_instance_variable(:@__global) if defined?(@__global)
      end

      def finalize
        singleton_class.def_delegators(:global, *public_instance_methods(false) - singleton_class.public_instance_methods(false))
      end
    end

    extends :definition, prepend: true do
      def method_added(name)
        if instance_variable_defined?(:@__finalized)
          remove_instance_variable(:@__finalized)
        end

        super
      end

      def method_missing(name, *args, &block)
        unless defined?(@__finalized)
          finalize
        end

        if public_method_defined?(name)
          global.__send__(name, *args, &block)
        else
          super
        end
      end

      if Module.private_method_defined?(:ruby2_keywords)
        ruby2_keywords(:method_missing)
      end

      def respond_to_missing?(name, *)
        public_method_defined?(name) || super
      end
    end

    applies do
      singleton_class.extend(Forwardable)
    end
  end
end
